@@ -10,18 +10,20 @@ from . import opt_smile_abc as smile
 class Bsm(opt.OptAnalyticABC):
     """
     Black-Scholes-Merton (BSM) model for option pricing.
-        Underlying price is assumed to follow geometric Brownian motion.
+        Underlying price is assumed to follow a geometric Brownian motion.
 
     Examples:
-        import pyfeng as pf
-        bsm = pf.BsmModel(sigma=0.2)
-        price = bsm.price(strike=105, spot=100, texp=1)
-        print(price)
-
-        sigma = np.array([0.2, 0.3, 0.5])
-        bsm = pf.BsmModel(sigma[:,None]) # sigma in axis=0
-        price = bsm.price(strike=np.array([90, 100, 110]), spot=100, texp=10, cp=np.array([-1,1,1]))
-        print(price)
+        >>> import numpy as np
+        >>> import pyfeng as pf
+        >>> m = pf.Bsm(sigma=0.2, intr=0.05, divr=0.1)
+        >>> m.price(np.arange(80, 121, 10), 100, 1.2)
+        array([15.71361973,  9.69250803,  5.52948546,  2.94558338,  1.48139131])
+        >>> sigma = np.array([0.2, 0.3, 0.5])[:, None]
+        >>> m = pf.Bsm(sigma, intr=0.05, divr=0.1) # sigma in axis=0
+        >>> m.price(np.array([90, 100, 110]), 100, 1.2, cp=np.array([-1,1,1]))
+        array([[ 5.75927238,  5.52948546,  2.94558338],
+               [ 9.4592961 ,  9.3881245 ,  6.45745004],
+               [16.812035  , 17.10541288, 14.10354768]])
     """
 
     @staticmethod
@@ -59,47 +61,40 @@ class Bsm(opt.OptAnalyticABC):
 
     def vega(self, strike, spot, texp, cp=1):
 
-        disc_fac = np.exp(-texp * self.intr)
-        fwd = spot * (1.0 if self.is_fwd else np.exp(-texp * self.divr)/disc_fac)
+        fwd, df, _ = self._fwd_factor(spot, texp)
 
         sigma_std = np.maximum(self.sigma * np.sqrt(texp), np.finfo(float).eps)
         d1 = np.log(fwd / strike) / sigma_std + 0.5 * sigma_std
 
-        vega = disc_fac * fwd * spst.norm.pdf(d1) * np.sqrt(texp)  # formula according to wikipedia
+        vega = df * fwd * spst.norm.pdf(d1) * np.sqrt(texp)  # formula according to wikipedia
         return vega
 
     def delta(self, strike, spot, texp, cp=1):
 
-        disc_fac = np.exp(-texp * self.intr)
-        div_fac = np.exp(-texp * self.divr)
-        fwd = spot * (1.0 if self.is_fwd else div_fac/disc_fac)
+        fwd, df, divf = self._fwd_factor(spot, texp)
 
         sigma_std = np.maximum(self.sigma * np.sqrt(texp), np.finfo(float).eps)
         d1 = np.log(fwd / strike) / sigma_std + 0.5 * sigma_std
 
         delta = cp * spst.norm.cdf(cp * d1)  # formula according to wikipedia
-        delta *= disc_fac if self.is_fwd else div_fac
+        delta *= df if self.is_fwd else divf
         return delta
 
     def gamma(self, strike, spot, texp, cp=1):
 
-        disc_fac = np.exp(-texp * self.intr)
-        div_fac = np.exp(-texp * self.divr)
-        fwd = spot * (1.0 if self.is_fwd else div_fac/disc_fac)
+        fwd, df, divf = self._fwd_factor(spot, texp)
 
         sigma_std = np.maximum(self.sigma * np.sqrt(texp), 100*np.finfo(float).eps)
         d1 = np.log(fwd / strike) / sigma_std + 0.5 * sigma_std
 
-        gamma = disc_fac * spst.norm.pdf(d1)/fwd/sigma_std  # formula according to wikipedia
+        gamma = df * spst.norm.pdf(d1)/fwd/sigma_std  # formula according to wikipedia
         if not self.is_fwd:
-            gamma *= (div_fac/disc_fac)**2
+            gamma *= (divf/df)**2
         return gamma
 
     def theta(self, strike, spot, texp, cp=1):
 
-        disc_fac = np.exp(-texp * self.intr)
-        div_fac = np.exp(-texp * self.divr)
-        fwd = spot * (1.0 if self.is_fwd else div_fac/disc_fac)
+        fwd, df, divf = self._fwd_factor(spot, texp)
 
         sigma_std = np.maximum(self.sigma * np.sqrt(texp), 100*np.finfo(float).eps)
         d1 = np.log(fwd / strike) / sigma_std
@@ -108,10 +103,10 @@ class Bsm(opt.OptAnalyticABC):
 
         # still not perfect; need to consider the derivative w.r.t. divr and is_fwd = True
         theta = -0.5*spst.norm.pdf(d1)*fwd*self.sigma/np.sqrt(texp) - cp*self.intr*strike*spst.norm.cdf(cp*d2)
-        theta *= disc_fac
+        theta *= df
         return theta
 
-    def impvol_newton(self, price, strike, spot, texp, cp=1, setval=False):
+    def _impvol_newton(self, price, strike, spot, texp, cp=1, setval=False):
         """
         BSM implied volatility with Newton's method
 
@@ -126,11 +121,10 @@ class Bsm(opt.OptAnalyticABC):
         Returns:
             implied volatility
         """
-        disc_fac = np.exp(-texp * self.intr)
-        fwd = spot * (1.0 if self.is_fwd else np.exp(-texp * self.divr)/disc_fac)
+        fwd, df, divf = self._fwd_factor(spot, texp)
 
         strike_std = strike / fwd  # strike / fwd
-        price_std = price / disc_fac / fwd  # forward price / fwd
+        price_std = price / df / fwd  # forward price / fwd
 
         bsm_model = Bsm(0, is_fwd=True)
         p_min = bsm_model.price(strike_std, 1.0, texp, cp)
@@ -187,7 +181,7 @@ class Bsm(opt.OptAnalyticABC):
         return _sigma
 
     #### Class method
-    impvol = impvol_newton
+    impvol = _impvol_newton
 
     def vol_smile(self, strike, spot, texp, cp=1, model='norm'):
         """
@@ -209,7 +203,7 @@ class Bsm(opt.OptAnalyticABC):
             price = self.price(strike, spot, texp, cp=cp)
             return norm.Norm(None).impvol(price, strike, spot, texp, cp=cp)
         elif model.lower() == 'norm-approx' or model.lower() == 'norm-grunspan':
-            fwd = spot * (1.0 if self.is_fwd else np.exp((self.intr - self.divr)*texp))
+            fwd, _, _ = self._fwd_factor(spot, texp)
             kk = strike / fwd
             lnk = np.log(kk)
             if model.lower() == 'norm-approx':
@@ -224,8 +218,7 @@ class Bsm(opt.OptAnalyticABC):
 
     def _price_suboptimal(self, strike, spot, texp, cp=1, strike2=None):
         strike2 = strike if strike2 is None else strike2
-        disc_fac = np.exp(-texp * self.intr)
-        fwd = spot * (1.0 if self.is_fwd else np.exp(-texp * self.divr) / disc_fac)
+        fwd, df, _ = self._fwd_factor(spot, texp)
 
         sigma_std = np.maximum(self.sigma * np.sqrt(texp), np.finfo(float).eps)
         d1 = np.log(fwd/strike2)/sigma_std
@@ -233,7 +226,7 @@ class Bsm(opt.OptAnalyticABC):
         d1 += 0.5*sigma_std
 
         price = fwd * spst.norm.cdf(cp * d1) - strike * spst.norm.cdf(cp * d2)
-        price *= cp*disc_fac
+        price *= cp*df
         return price
 
     def _barrier_params(self, barrier, spot):
@@ -398,14 +391,14 @@ class BsmDisp(smile.OptSmileABC):
             volatility smile under the specified model
         """
         if model.lower() == 'norm-approx':
-            fwd = spot * (1.0 if self.is_fwd else np.exp((self.divr - self.intr)*texp))
+            fwd, _, _ = self._fwd_factor(spot, texp)
             kkd = self.disp(strike) / self.disp(fwd)
             lnkd = np.log(kkd)
             sig_beta = self.beta * self.sigma
             vol = self.sigma * self.disp(fwd) * np.sqrt(kkd) * (1 + lnkd ** 2 / 24) / \
                 (1 + sig_beta ** 2 * texp / 24)
         elif model.lower() == 'bsm-approx':
-            fwd = spot * (1.0 if self.is_fwd else np.exp((self.divr - self.intr)*texp))
+            fwd, _, _ = self._fwd_factor(spot, texp)
             kk = strike / fwd
             lnk = np.log(kk)
             fwdd = self.disp(fwd)
